function ApiConnector(fallback_callback){
  /*
    if(arguments.callee._singletonInstance){
	    return arguments.callee._singletonInstance;
  	}
  	arguments.callee._singletonInstance = this;
  */
  	var self = this; self.name = "ApiConnector"; // MANDATORY

    this.fallback = undefined;
    if(typeof(fallback_callback) !== "undefined"){
      this.fallback = fallback_callback;
    }
    this.protocol = "https://";
    this.secure_connection = true;
    this.player_id = 0;
    this.api_key = "";
    this.xhr = XMLHttpRequest;
    this.api_options = {};
    this.proxy_options = false;

    this.last_success_timestamp = Date.now();
    this.MAX_HTTP_TIMEOUT = 60000; // 1min

    this.setCoreId = function(id){
      if(typeof(id) !== "undefined"){
        if(!isNaN(parseInt(id))){
          self.player_id = parseInt(id);
        }
      }
    }

    this.setProxyOptions = function(options){
      self.proxy_options = options;
    };

    this.getInfo = function(){
      var info = {api_options:self.api_options,proxy_options:self.proxy_options};
      return info;
    };

    this.setSecureConnection = function(activate){
      if(typeof(activate) === "boolean"){
        self.secure_connection = activate;
      }
      /*
      try{
        this.xhr = null;
        this.xhr = XMLHttpRequest;
      }catch(e){
        console.log("could not restart XMLHttpRequest class");
      }
      */
      if(self.secure_connection){
        self.protocol = "https://";
      }else{
        self.protocol = "http://";
      }

    }

    this.setApiKey = function(key){
      if(typeof(key) !== "undefined"){
          self.api_key = key;
      }
    };

    this.buildUrl = function(api_url,api_ver,method){
        self.api_options = {api_server:api_url,api_ver:api_ver,protocol:self.protocol}
        return self.protocol +api_url + "/" + api_ver + "/" + method;
    };

    this.post = function(api_url,api_ver,method, data, callback, params){
        var url = self.buildUrl(api_url,api_ver,method);
        self.makeRequest("POST",url, function(response, status){
            if(typeof(callback) !== "undefined"){
                try{
                    callback(JSON.parse(response),status,params);
                }catch(e){
                    callback(response,status,params);
                }
            }
        },data);
        url = null;
    };

    this.get = function(api_url,api_ver,method, data, callback, params){
        var url = self.buildUrl(api_url,api_ver,method);
        self.makeRequest("GET",url+"/"+data,function(response, status){
            if(typeof(callback) !== "undefined"){
                try{
                    callback(JSON.parse(response),status,params);
                }catch(e){
                    callback(response,status,params);
                }
            }
        });
        url = null;
    };

    this.makeRequest = function(method,url,callback,data){
        var xhr = new self.xhr();
        url = url+"?"+Date.now();
        xhr.open(method, url, true);
        xhr.setRequestHeader("From", "player-"+self.player_id);
        xhr.setRequestHeader("apikey", "API-"+self.player_id+"-"+self.api_key);
        if(typeof(data) === "object"){
            if(!(data instanceof FormData)){
              data = JSON.stringify(data);
              xhr.setRequestHeader('Content-Type', 'application/json; charset=utf-8');
            }
        }else{
            xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded; charset=utf-8');
        }
        xhr.addEventListener("load", function () {
            if(xhr.status === 200 || xhr.status === 0){
                callback(xhr.response,"success");
            }else{
              if(Date.now() - self.last_success_timestamp > self.MAX_HTTP_TIMEOUT){
                self.last_success_timestamp = Date.now();
                self.fallback(self.api_options);
              }
              callback(null,"error");
            }
        }, false);
        /*
        xhr.addEventListener("progress", function (e) {
            if(e.lengthComputable){
                var percent = (e.loaded / e.total) * 100;
                console.log(percent+" % of "+e.total+" bytes");
            }
        }, false);
        */
        xhr.addEventListener("error", function (e) {
            try{
              if(Date.now() - self.last_success_timestamp > self.MAX_HTTP_TIMEOUT){
                self.last_success_timestamp = Date.now();
                self.fallback(self.api_options);
              }
              callback(null,"error");
            }catch(e){
              callback(null,"exception");
            }
        }, false);
        xhr.addEventListener("abort", function (e) {
          if(Date.now() - self.last_success_timestamp > self.MAX_HTTP_TIMEOUT){
            self.last_success_timestamp = Date.now();
            self.fallback(self.api_options);
          }
          callback(null,"error");
        }, false);
        if(typeof(data) !== "undefined"){
            try{
              xhr.send(data);
            }catch(e){
              console.log(e);
              if(Date.now() - self.last_success_timestamp > self.MAX_HTTP_TIMEOUT){
                self.last_success_timestamp = Date.now();
                self.fallback(self.api_options);
              }
              callback(null,"exception");
            }
        }else{
            try{
              xhr.send();
            }catch(e){
              console.log(e);
              if(Date.now() - self.last_success_timestamp > self.MAX_HTTP_TIMEOUT){
                self.last_success_timestamp = Date.now();
                self.fallback(self.api_options);
              }
              callback(null,"exception");
            }
        }
    }
}
